czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
c       Program SURF_PERT
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
c-----------------------------------------------------------------------
c       This program will accept one liquid layer at the surface,
c       in wich case ellipticity of Rayleigh wave is that at the
c       top of solid array. Love wave calculations ignore liquid layer.
c       The root determination section is one of interval halving
c       once a zero crossing has been found.
c       To follow modes the program initially finds the phase velocities
c       of the 'mode' number of modes for period t1, then it finds the
c       phase velocities for period t1+dt using previous results to stay
c       on the same mode. It is preferable 'dc' positive since the
c       program will follow a mode up to its cutoff value. For 'dc'
c       negative the program cannot pick up any modes.
c       It optionally  accounts for attenuation and produces group velocity
c       derivatives by layer parameters;
c       It includes sphericity corrections by default
c       Love waves and Rayleigh waves are treated separately
c-----------------------------------------------------------------------
C         INPUT ARGUMENTS:
C-------infile outfile wavetype kmin kmax tmin tmax dt [-a] [-f] [-d par_type  nl d_par] [-e[1/2]] [-n]
C	 a. NECESSARY ARGUMENTS
C-------infile : input parameters' file name----------------------------
C-------outfile: output file name----------------------------
C-------wavetype: R/L
C-------kmin,kmax - radial orders of modes to be computed
C-------tmin, tmax, tstep -  the range of periods and increment are defined
C        b. OPTIONAL  ARGUMENTS
C-------[-a] if present, attenuation is included; 
C-------[-f] if present, flattenning is included; 
C-------[-d par_type nl d_par] partial derivatives of phase and group velocities
C-------by parameter "par_type" (vp/vs/rho/thick) of the nl-th layer would be 
C-------found using +/- dx absolute perturbation.
C-------[-e[1/2/n]] if -e is present, eigenfunctions will be found; if 1 or 2 
C-------are present after -e 1st/1st and 2nd derivatives by depth will be found.
C-------if n is present after e, eigenfunctions are normalized by their maxima 
C         INPUT FILE: 
C-------file "infile" with model data and parameters of calculations; 
C-------see an example in the README file.
C         OUTPUT FILES: 
C-------grv.file : output group velocity xy-file;filename=outfile//".R.grv" or
C-------outfile//".L.grv" depending on wavetype.
c-----------------------------------------------------------------------
C-------phv.file : output phase velocity xy-file;filename=outfile//".R.phv" or
C-------outfile//".L.phv" depending on wavetype.
c-----------------------------------------------------------------------
C-------att.file : output apparent Q file;filename=outfile//".R.att" or
C-------outfile//".L.att" depending on wavetype. This file will appear only withC-------[-a] option!
c-----------------------------------------------------------------------
C-------eigen.file: output normalized eigenfunction as xy-file;
C-------filename=outfile//".R_VER.eig" and outfile//".R_HOR.eig" or 
C-------outfile//".L.eig". All periods are represented in one file:
C-------each period sequence starts from "T=",period value and number of depth
C-------This file will appear only with [-e] option!
c-----------------------------------------------------------------------
C-------outfile: general output file containing eigenfunctions, integrals,
C-------lagrangian, phase velocity partial derivatives, eigenfunctions 
C-------derivatives by depth, etc; to be specified by the option in the input 
C-------file (see README).
c-----------------------------------------------------------------------
        parameter (nsize=3000,nper=1000)
	common/d/  a(nsize),b(nsize),rho(nsize),d(nsize),qs(nsize)
        common/o/  c(nper,10),t(nper),ratio(nper,10)
        common/c/nmax,mmax,kmax,idrop,iedit,ndiv,mode,fact
        common/newly/ t1,dt,c1,dc,lstop,iq,istru,cinit
        common/rar/ depth(nsize),amprfi(nsize),ampz(nsize),strz(nsize),
     *  strrfi(nsize),mmm
        common/rar1/ dcda,dcdb,dcdr,dwx,g1(nsize),g2(nsize)
        common/rco/ ccc,cvar,ugr,wvno,rat,arle
        common/rco1/ sumi0,sumi1,sumi2,sumi3,flagr
        common/deriv/ nderiv,ndpth,dpth(nsize),dspz(nsize),dsprfi(nsize),
     *  drvz(5,nsize),drvrfi(5,nsize)
        common/log/ KEY_ATTEN,KEY_FLAT,KEY_DERIV,KEY_EIGEN,KEY_EIG_NORM,KEY_EIGEN_DER1,KEY_EIGEN_DER2,KEY_VP,KEY_VS,KEY_TH,KEY_RHO
	common/i_nit/KEY_CINIT
	common/ref/ a_ref(nsize),b_ref(nsize),rho_ref(nsize),d_ref(nsize),qs_ref(nsize)
C-------------------------------------------------------------------------
        real*8 dcda(nsize),dcdb(nsize),dcdr(nsize),dwx(nsize)
        logical KEY_DERIV,KEY_RHO,KEY_TH,KEY_VP,KEY_VS,KEY_ATTEN,KEY_FLAT
	logical KEY_EIGEN,KEY_EIG_NORM,KEY_EIGEN_DER1,KEY_EIGEN_DER2,KEY_CINIT
        integer imax(10)
C-------------------------------------------------------------------------
c       fact - number of wavelenghts below first layer where phase
c       velocity is less than shear velocity that we may consider
c       to be effective halfspace.
	data pi/3.1415927/,t_base/1.0/
	call INIT(dx,nlay_deriv,idispr,idispl,kind)
	   ndiv_store=ndiv
	   t1_store=t1
	   mmax_store=mmax
c----------------------------------------------------------------------
c       kmax - number of periods; t1 - initial starting period;
c       dt - period increment; c1 - initial phase velocity guess;
c       dc - phase velocity increment; istru<1 - depth dependent
c       parameters will be calculated in midpoints of sublayers,
c       else program STRUT will be called.
c----------------------------------------------------------------------
c       nderiv - highest order of derivative of eigenfunction;
c       ndpth - number of depthes for wich eigenfunction and its
c       derivatives are desired.
c----------------------------------------------------------------------
c       dpth(j) - array of depths for calculation of eigenfunction
c       and its derivatives.
c----------------------------------------------------------------------
C---------computations start-------------------------------------------
C##########################################################################
1     ncount=ncount+1
	ndiv=ndiv_store
	mmax=mmax_store
	t1=t1_store
        do i=1,nsize
	depth(i)=0.0
	enddo
	do i=1,nper
	do j=1,10    
	c(i,j)=0.0
	ratio(i,j)=0.0
	enddo
	enddo
c----------------------------------------------------------------------
c       ndiv - number of subdivisions for one layer;
c       mode - number of modes for which dispersion curves are desired;
c !!!       iedit = 0 - only dispersion, iedit = 1 - dispersion, amplitudes
c       of eigenfunctions and derivatives of phase velocity,
c----------------------------------------------------------------------
C----------------for perturbations--------------------------------S
	 if(KEY_DERIV)                        then
	       if(ncount.eq.2) then
        if(KEY_TH) d_ref(nlay_deriv)=d_ref(nlay_deriv)-dx
        if(KEY_VP) a_ref(nlay_deriv)=a_ref(nlay_deriv)-dx
        if(KEY_VS) b_ref(nlay_deriv)=b_ref(nlay_deriv)-dx
        if(KEY_RHO) rho_ref(nlay_deriv)=rho_ref(nlay_deriv)-dx
 	KEY_EIGEN=.false.
	                       endif
	       if(ncount.eq.3) then
        if(KEY_TH) d_ref(nlay_deriv)=d_ref(nlay_deriv)+2*dx
        if(KEY_VP) a_ref(nlay_deriv)=a_ref(nlay_deriv)+2*dx
        if(KEY_VS) b_ref(nlay_deriv)=b_ref(nlay_deriv)+2*dx
        if(KEY_RHO) rho_ref(nlay_deriv)=rho_ref(nlay_deriv)+2*dx
		               endif
		                              endif
         if(KEY_FLAT.and..not.KEY_ATTEN)then
C--------flattening without attenuation-------------------------------
	 call flat1(d_ref,rho_ref,a_ref,b_ref,d,rho,a,b,mmax,kind)
	                                endif
         if(.not.KEY_FLAT.and..not.KEY_ATTEN)then
C---------no flattening and no  attennuation-----------------
	       do i=1,mmax
	      d(i)=d_ref(i)
	      a(i)=a_ref(i)
	      b(i)=b_ref(i)
	      rho(i)=rho_ref(i)
	       enddo
	                              endif
c----------------------------------------------------------------------
        nmax=mmax
        number=0
        numbel=0
	lstop=0
	idrop=0
	iq=0
	imax(1)=0
	ilay=1
	if(b_ref(1).eq.0.0) ilay=2
	b_corr=0.0
	if(KEY_ATTEN) b_corr=qs_ref(ilay)*alog(t_base/t1)/pi
	qq=b_ref(ilay)
	if(kind.eq.2)qq=0.9*qq
	if(KEY_CINIT)qq=cinit
	c1=qq*(1.+b_corr)
 	if(ilay.eq.2.and.kind.eq.2)c1=1.5
	print *,'START c1=',c1
C-----------------------start of main calculations------------
C       if(KEY_ATTEN.and.kind.eq.1) print*,'   PERIOD (s)    ALPHA_L (1/1000km)           Q_L '
C       if(KEY_ATTEN.and.kind.eq.2) print*,'   PERIOD (s)    ALPHA_R (1/1000km)           Q_R '
        call calcul(ncount,dx,imax,idispr,idispl,numbel,number,kind,t_base,*1)
	      STOP 'ALL IS DONE'
                              END
        subroutine calcul(ncount,dx,imax,idispr,idispl,number,numbel,kind,t_base,*)
C--------------------------------------------------------------------
        parameter (nsize=3000,nper=1000)
	common/d/  a(nsize),b(nsize),rho(nsize),d(nsize),qs(nsize)
        common/o/  c(nper,10),t(nper),ratio(nper,10)
        common/c/nmax,mmax,kmax,idrop,iedit,ndiv,mode,fact
        common/newly/ t1,dt,c1,dc,lstop,iq,istru
        common/rar/ depth(nsize),amprfi(nsize),ampz(nsize),strz(nsize),
     *  strrfi(nsize),mmm
        common/rar1/ dcda,dcdb,dcdr,dwx,g1(nsize),g2(nsize)
        common/rco/ ccc,cvar,ugr,wvno,rat,arle
        common/rco1/ sumi0,sumi1,sumi2,sumi3,flagr
        common/deriv/ nderiv,ndpth,dpth(nsize),dspz(nsize),dsprfi(nsize),
     *  drvz(5,nsize),drvrfi(5,nsize)
        common/log/ KEY_ATTEN,KEY_FLAT,KEY_DERIV,KEY_EIGEN,KEY_EIG_NORM,KEY_EIGEN_DER1,KEY_EIGEN_DER2
	common/ref/ a_ref(nsize),b_ref(nsize),rho_ref(nsize),d_ref(nsize),qs_ref(nsize),dpth_ref(nsize)
C-------------------------------------------------------------------------
        real*8 dcda(nsize),dcdb(nsize),dcdr(nsize),dwx(nsize)
        logical KEY_DERIV,KEY_ATTEN,KEY_FLAT,KEY_EIGEN,KEY_EIG_NORM
	real*4 ugr_plus(nper),ugr_minus(nper),ugr0(nper)
	real*4 ccc_plus(nper),ccc_minus(nper),ccc0(nper)
        integer imax(10)
C-------------------------------------------------------------------------
	data pi/3.1415927/,R0/6371.0/,dep_lim/1200.0/
C-----------------------FORMATS-------------------------------------------
11      format(a,/2(6x,i3,2x),6x,i2,8x,i1)
44      format(1x,/(4f10.4))
55      format(10x,3f10.4)
12      format(2(7x,i1,2x),5x,f3.0)
1000    format(5x,i3,2x,4(5x,f6.0,2x),6x,i1)
1001    format(1x,/(8f10.0))
1002    format(a)
1004    format(34x,'Love mode ',i2)
1005    format(32x,'Rayleigh mode ',i2)
1006    format(8x,i2,9x,i3)
1007    format(1x,/(7f10.0))
5000    format(1x,/1x,'t=',f6.2,2x,'c=',f6.4,2x,'cvar=',f6.4,2x,'ugr=',
     *  f6.4,2x,'wvnumb=',e11.4,2x,'al=',e11.4)
5002    format(1x,/3x,'I0 =',e11.4,5x,'I1 =',e11.4,5x,'I2 =',e11.4,
     *  5x,'L =',e11.4)
5004    format(1x,/4x,'m',5x,'depth',7x,'disp',7x,'stress',8x,
     *  'dc/db',8x,'dc/dr',10x,'g')
5005    format(2x,i3,f10.2,5e13.4)
5006    format(1x,/1x,'t=',f6.2,2x,'c=',f6.4,2x,'cvar=',f6.4,2x,'ugr=',
     *  f6.4,2x,'wvnumb=',e11.4,2x,'ar=',e11.4,/36x,'ratio =',e10.4)
5007    format(1x,'I0 =',e10.4,2x,'I1 =',e10.4,2x,'I2 =',e10.4,
     *  2x,'I3 =',e10.4,2x,'L =',e10.4)
5008    format(1x,/4x,'m',5x,'depth',7x,'dispr',8x,'dispz',
     *  7x,'stresr',7x,'stresz',8x,'dwx')
5009    format(1x,/4x,'m',5x,'depth',7x,'dcda',9x,'dcdb',9x,
     *  'dcdr',10x,'g1',11x,'g2')
5010    format(1x,/10x,
     *  'Eigenfunction V (n=0) and its derivatives with respect to z',
     *  /25x,'(n - the order of derivative)',//2x,'depth\\n',
     *  6x,'0',11x,'1',11x,'2',11x,'3',11x,'4',11x,'5')
5020    format(1x,/10x,
     *  'Eigenfunction Vr (n=0) and its derivatives with respect to z',
     *  /25x,'(n - the order of derivative)',//2x,'depth\\n',
     *  6x,'0',11x,'1',11x,'2',11x,'3',11x,'4',11x,'5')
5030    format(1x,/10x,
     *  'Eigenfunction Vz (n=0) and its derivatives with respect to z',
     *  /25x,'(n - the order of derivative)',//2x,'depth\\n',
     *  6x,'0',11x,'1',11x,'2',11x,'3',11x,'4',11x,'5')
5011    format(1x,f7.2,6e12.4)
3007    format(1x,/2x,'t / mode',3x,'1',6x,'2',6x,'3',6x,'4',6x,'5',
     *  6x,'6',6x,'7',6x,'8',6x,'9',6x,'10',/)
3008    format(1x,f6.2,3x,10f7.3)
3009    format(1x,/11x,'Mode ',i2,',  period',f6.2,
     *  'sec.  Subroutine Nevill. Too many cycles.')
      R0_invsq=1./R0**2
      PRint*, 'KEY_ATTEN=',KEY_ATTEN
      PRint*, 'KEY_FLAT =',KEY_FLAT 
      PRint*, 'KEY_EIGEN=',KEY_EIGEN
      PRint*, 'KEY_EIG_N=',KEY_EIG_NORM
      PRint*, 'KEY_DERIV=',KEY_DERIV
C--------------INITIATION-------------------------------------------
        cc=c1
                     ifunc=kind                  
                                        continue
        c1=cc
c
c
        kmode=mode
                                do 2 i=1,10
2       imax(i)=0
                                do 9998 k=1,kmax
        t1=t(k)
	if (KEY_ATTEN) then
C----------------current model for a given t1---------------------
	do i=1,mmax
	b(i)=b_ref(i)*(1.+qs_ref(i)*alog(t_base/t1)/pi)
	qp=qs_ref(i)*1.33333333*b_ref(i)**2/a_ref(i)**2
	a(i)=a_ref(i)*(1.+qp*alog(t_base/t1)/pi)
	rho(i)=rho_ref(i)
	d(i)=d_ref(i)
	enddo
         if(KEY_FLAT)call flat1(d,rho,a,b,d,rho,a,b,mmax,kind)
	endif
                                do 9997 iq=1,kmode
                        if(k-1) 605,605,599
599                     if(iq-2) 600,601,601
c600     c1=c(k-1,1)-0.1 
600     c1=0.90*c(k-1,1)
                                                        goto 605
601                     if(dc*(c(k-1,iq)-c(k,iq-1))) 602,603,604
602     c1=c(k,iq-1)+0.01*dc
                                                        goto 605
603     c1=c(k,iq-1)+0.01*dc
                                                        goto 605
604     c1=c(k-1,iq)
c!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
c       There were c1=c(k-1,iq)-0.1
c!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
605                                     continue
        idrop=0
999     del1=dltar(c1,t1,ifunc)
80      c2=c1+dc
        idrop=0
        del2=dltar(c2,t1,ifunc)
                    if(sign(1.,del1).ne.sign(1.,del2))  goto 54
81	  c1=c2
        del1=del2
                        if(c1-0.8*b(1)) 250,251,251
251                     if(c1-(b(mmax)+0.3)) 252,250,250
252                                                     goto 80
54                  continue         
        call nevill (t1,c1,c2,del1,del2,ifunc,cn)
                        if(lstop.eq.0)                  go to 3006
        lstop=0
                        if(k.eq.1.and.iq.eq.1)          go to 3005
        if(ncount.eq.1)write(2,3007)
                        if(k.eq.1)                      go to 3004
                                do 3003 i=1,k-1
                                do 3000 j=1,mode
                        if(i.gt.imax(j))                go to 3001
3000                                    continue
        jmax=mode
                                                        go to 3002
3001    jmax=j-1
3002     continue
3003                                    continue
3004                    if(iq.eq.1)                     go to 3005
        jmax=iq-1
3005    continue 
	if(ncount.eq.1)write(2,*)iq,t(k)
                                                        go to 9999
3006    c1=cn
                        if(c1-b(mmax)) 121,121,250
121     c(k,iq)=c1
                                                goto (6001,6002),ifunc
6002                                    continue
        ratio(k,iq)=dltar(c1,t1,3)
                                                        goto 6003
6001                                    continue
6003                                    continue
        c1=c1+0.01*dc
        imax(iq)=k
                                                        goto 9997
250                     if(k*iq-1) 256,256,255
256     continue 
         if(ncount.eq.1)write(2,258)
258     format(1x,/22x,'Improper initial value. No zero found')
                                                        goto 9999
9997                                    continue
C--------------------search of root is finished--------------------
                                                        goto 9998
255     kmode=iq-1
                        if(kmode) 9996,9996,9998
9998                                    continue
9996                                    continue
        mmax=nmax
                                do 9995 iq=1,mode
        j=imax(iq)
                        if(j) 9995,9995,9994
9994                                            goto (7001,7002),ifunc
C------------------LOVE     WAVE PART------------------------------S
7001                                    continue
        number=-1
        if(ncount.eq.1)write(2,1004)iq
	print *,'MODE=',iq-1,';  NUMBER OF PERIODS IN OUTPUT=',j
                                do 8889 lip=1,j
	if(KEY_ATTEN)then
	do i=1,mmax
	b(i)=b_ref(i)*(1.+qs_ref(i)*alog(t_base/t(lip))/pi)
	qp=qs_ref(i)*1.33333333*b_ref(i)**2/a_ref(i)**2
	a(i)=a_ref(i)*(1.+qp*alog(t_base/t(lip))/pi)
	d(i)=d_ref(i)
	rho(i)=rho_ref(i)
	qs(i)=qs_ref(i)
	enddo
         if(KEY_FLAT)call flat1(d,rho,a,b,d,rho,a,b,mmax,kind)
	endif
        call leigen(numbel,lip,iq)
        numbel=numbel+1
C------------ATTENUATION OUTPUT---------------S
        qL_app=20000.
	if(KEY_ATTEN) then
	     skd=0.0
	     do i=1,mmm
	     skd=skd+dcdb(i+1)*b(i)*qs(i)
	     enddo
	   alphL=pi/t(lip)*skd/ccc/ccc
	   qL_app= pi/alphL/ugr/t(lip)
C       print '(2(F10.3,6X),10X,F10.3)',t(lip),alphL*1000.,qL_app
           endif
	if(ncount.eq.1)then
	write(2,*)'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@'
        if(.not.KEY_FLAT)write(2,'(6(E14.7,2X))')t(lip),ccc,ugr,wvno,arle,qL_app
        if(KEY_FLAT)write(2,'(6(E14.7,2X))')t(lip),ccc,ugr,sqrt(wvno**2-R0_invsq),arle,qL_app
	write(22,*)t(lip),ugr
	write(21,*)t(lip),ccc,cvar
        write(2,*)sumi0,sumi1,sumi2,flagr
        if(KEY_ATTEN)write(29,*) t(lip),qL_app
	ugr0(lip)=ugr
	ccc0(lip)=ccc
		       endif
        if(ncount.eq.2)then
	ugr_minus(lip)=ugr
	ccc_minus(lip)=ccc
		       endif
	if(ncount.eq.3)then
	   ugr_plus(lip)=ugr
	   ccc_plus(lip)=ccc
		       endif
                        if(iedit.eq.0)                  go to 8889
                        if(iedit.eq.2)                  go to 8890
C       if(ncount.eq.1)write(2,5005)(m,depth(m),amprfi(m),strrfi(m),dcdb(m),dcdr(m),g1(m),m=1,mmm)
                                                        go to 8889
8890    if(ncount.eq.1.and.KEY_EIGEN) then
        IF(KEY_EIG_NORM) then
	bmax=0.0
	do i=1,nmax
        if(abs(amprfi(i)).gt.bmax)bmax=abs(amprfi(i))
	enddo
	do i=1,nmax
	amprfi(i)=amprfi(i)/bmax
	enddo
			 endif
        write (27,*)amprfi(1), z(R0,depth(1),KEY_FLAT),' T(s)=',t(lip)
                                do  m=2,nmax 
        if(amprfi(m).eq.0.0) go to 3333
        write (27,*)amprfi(m),  z(R0,depth(m),KEY_FLAT)
                                   enddo         
        write (27,'(/)'  )
                   endif
3333			 do 5012 m=1,ndpth
C        if(abs(dsprfi(m)).lt.0.1E-15)go to 5012
         if(abs(dsprfi(m)).lt.0.1E-15.or.dpth_ref(m).gt.dep_lim*(1.+float(iq-1)/3.).or.dpth_ref(m).gt.2900.)go to 5012
        if(.not.KEY_FLAT)write(2,'(5(E14.7,2X))')dpth_ref(m),dsprfi(m),(drvrfi(lm,m),lm=1,nderiv)
	if(KEY_FLAT)write(2,'(5(E14.7,2X))')dpth_ref(m),dsprfi(m)*(1.-dpth_ref(m)/R0),drvrfi(1,m)-dsprfi(m)/R0
5012			 enddo
8889                                    continue
C------------------LOVE     WAVE PART------------------------------E
                                                        go to 3335
C------------------RAYLEIGH WAVE PART------------------------------S
7002                    if(ncount.eq.1)write(2,1005)iq
	print *,'MODE=',iq-1,';  NUMBER OF PERIODS IN  OUTPUT=',j
                                do 8888 lip=1,j
	if(KEY_ATTEN)then
	do i=1,mmax
	b(i)=b_ref(i)*(1.+qs_ref(i)*alog(t_base/t(lip))/pi)
	qp=qs_ref(i)*1.33333333*b_ref(i)**2/a_ref(i)**2
	a(i)=a_ref(i)*(1.+qp*alog(t_base/t(lip))/pi)
	d(i)=d_ref(i)
	rho(i)=rho_ref(i)
	qs(i)=qs_ref(i)
	enddo
         if(KEY_FLAT)call flat1(d,rho,a,b,d,rho,a,b,mmax,kind)
	endif
        call reigen(number,lip,iq)
                        if(number.eq.-1) number=0
        number=number+1
	qR_app=20000.
	if(KEY_ATTEN) then
        skd=0.0
	do i=1,mmm
	skd=skd+dwx(i+1)*qs(i)
	enddo
	alphR=pi/t(lip)*skd/ccc/ccc
	qR_app= pi/alphR/ugr/t(lip)
C	print '(2(F10.3,6X),10X,F10.3))',t(lip),alphR*1000.,qR_app
            endif
	if(ncount.eq.1)then
	write(2,*)'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@'
	if(.not.KEY_FLAT)write(2,'(7(E14.7,2X))')t(lip),ccc,ugr,wvno,arle,rat,qR_app
	if(KEY_FLAT)WRite(2,'(7(E14.7,2X))')t(lip),ccc,ugr,sqrt(wvno**2-R0_invsq),arle,rat,qR_app
	write(22,*)t(lip),ugr
        if(KEY_ATTEN)write(29,*)t(lip),qR_app   
	ugr0(lip)=ugr
	ccc0(lip)=ccc
		end if
	if(ncount.eq.2)then 
	ugr_minus(lip)=ugr
	ccc_minus(lip)=ccc
	         endif
        if(ncount.eq.3)then
	ugr_plus(lip)=ugr
	ccc_plus(lip)=ccc
	         endif
	if(ncount.eq.1)write(21,*)t(lip),ccc,cvar
        if(ncount.eq.1)write(2,*)sumi0,sumi1,sumi2,sumi3,flagr
                        if(iedit.eq.0)                  go to 8888
                        if(iedit.eq.2)                  go to 8891
C       if(ncount.eq.1)write(2,5005)(m,depth(m),amprfi(m),ampz(m),strrfi(m),strz(m),
C    *  dwx(m),m=1,mmm)
C       if(ncount.eq.1)write(2,5005)(m,depth(m),dcda(m),dcdb(m),dcdr(m),g1(m),
C    *  g2(m),m=1,mmm)
C                                                       go to 8888
8891    if(ncount.eq.1.and.KEY_EIGEN) then
        IF(KEY_EIG_NORM) then
	bmax=0.0
	cmax=0.0
	do i=1,nmax
        if(abs(amprfi(i)).gt.bmax)bmax=abs(amprfi(i))
        if(abs(ampz(i)).gt.cmax)cmax=abs(ampz(i))
	enddo
	do i=1,nmax
	amprfi(i)=amprfi(i)/bmax
	ampz(i)=ampz(i)/cmax
	enddo
			 endif
 	write(27,*) amprfi(1),z(R0,depth(1),KEY_FLAT), ' T(s) =',t(lip)
 	write(28,*) ampz(1),  z(R0,depth(1),KEY_FLAT), ' T(s) =',t(lip)
                                do  m=2,nmax 
	if(amprfi(m).eq.0.0.or.ampz(m).eq.0.0) go to 3334
 	write(27,*) amprfi(m),z(R0,depth(m),KEY_FLAT)
 	write(28,*) ampz(m),z(R0,depth(m),KEY_FLAT)
				enddo
3334    write(27,'(/)')
	write(28,'(/)')
                                      endif
			        do 5013 m=1,ndpth
C       if(abs(dsprfi(m)).lt.0.1E-15)then
         if(abs(dsprfi(m)).lt.0.1E-15.or.dpth_ref(m).gt.dep_lim*(1.+float(iq-1)/3.).or.dpth_ref(m).gt.2900.)then
C       PRint*,m,dpth_ref(m),dsprfi(m),dsprfi(m)*(1.-dpth_ref(m)/R0)
        go to 4444
                                     endif
C       PRint*,ndpth,dpth_ref(m),dsprfi(m),dsprfi(m)*(1.-dpth_ref(m)/R0)
        if(.not.KEY_FLAT)write(2,'(5(E14.7,2X))')dpth_ref(m),dsprfi(m),(drvrfi(lm,m),lm=1,nderiv)
	if(KEY_FLAT)write(2,'(5(E14.7,2X))')dpth_ref(m),dsprfi(m)*(1.-dpth_ref(m)/R0),drvrfi(1,m)-dsprfi(m)/R0
5013                                    continue
4444    mend=m-1
        write(2,*)'$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$',mend
C                               do 5014 m=1,ndpth
                                do 5014 m=1,mend 
C       if(abs(dsprfi(m)).lt.0.1E-15)go to 8888
       if(abs(dsprfi(m)).lt.0.1E-15.or.dpth_ref(m).gt.dep_lim*(1.+float(iq-1)/3.).or.dpth_ref(m).gt.2900.)go to 8888
        if(.not.KEY_FLAT)write(2,'(5(E14.7,2X))')dpth_ref(m),dspz(m),(drvz(lm,m),lm=1,nderiv)
	if(KEY_FLAT)write(2,'(5(E14.7,2X))')dpth_ref(m),dspz(m)*(1.-dpth_ref(m)/R0),drvz(1,m)-dspz(m)/R0
5014                                    continue
8888                                    continue
C------------------RAYLEIGH WAVE PART------------------------------E
3335        write(21,'(/)')
            write(22,'(/)')
         if(KEY_ATTEN)write(29,'(/)')
9995	continue
9999                                    continue
                 if(ncount.lt.3.and.KEY_DERIV)      return1         
777           if(KEY_DERIV) then
	      call qderiv(ugr_minus,ugr0,ugr_plus,t,kmax,dx,25)
            write(25,'(/)')
            write(26,'(/)')
	      call qderiv(ccc_minus,ccc0,ccc_plus,t,kmax,dx,23)
            write(23,'(/)')
            write(24,'(/)')
		   return 
			    endif
                                                                END
C#########################################################################
	    subroutine qderiv(u1,u2,u3,t,nper,dx,nfile)
	    real*4 u1(2),u2(2),u3(2),t(2)
	    do i=1,nper
	    du1=(u3(i)-u1(i))/2./dx
	    du2=(u1(i)+u3(i)-2.*u2(i))/dx**2
	    write(nfile,*)t(i),du1
	    write(nfile+1,*)t(i),du2
	    enddo
	    return
            end
C#########################################################################
            function z(R0,dep,KEY)
C----------to return a  real depth if flattenning was applied
	    logical KEY
	    z=dep
	    if(.not.KEY)return
	    z=R0*(1.-exp(-dep/R0))
	    return
	    end
c#################################################################
      subroutine flat1(h,ro,vp,vs,h1,ro1,vp1,vs1,n,kind)
c     subroutine applies earth-flattening corrections by
c     scaling the model.for reference see Biswas 1972.
c     kind=1 for Love wave calculations
c     kind=2 for Rayleigh wave calculations
c     h -is thickness
c-input h,ro,vp,vs,n,kind; output h1,ro1,vp1,vs1.
C     TRANSFORMATION
C-----R(i)-radius of i-th boundary (free surfase is ignored)
C-----h(i) by R0*(ln(R0/R(i))-ln(R0/R(i-1));R(0)=R0
C-----vs,vp mult by factor "dif"=R0*(1/R(i+1)-1/R(i))/ln(R(i)/R(i+1))
C-----ro  mult by factor (R(i)**pwr-R(i+1)**pwr)/(ln(R(i)/R(i+1))*pwr*R0**pwr)
C-----for Rayleigh pwr=2.275 
C-----for Love     pwr=5.000  
      real*4 h(2),ro(2),vp(2),vs(2),hh(3000)
      real*4 h1(2),ro1(2),vp1(2),vs1(2)
      data a/6371.0/
      do i=1,n
      hh(i)=h(i)
      enddo
      pwr=2.2750
C     pwr=1.2750
      if(kind.eq.1) pwr=5.0
      nm=n-1
      hs=0.0
C-----transfer thickness in radius------
      do 5 i=1,n
      ht=hs
      hs=hs+hh(i)
    5 hh(i)=a-ht
C-----now hh(i) are radii  (starting from the first boundary)
c -layers' scaling
      do 10 i=1,nm
      ii=i+1
      fltd=alog(hh(i)/hh(ii))
      dif=(1.0/hh(ii)-1.0/hh(i))*a/fltd
      difr=hh(i)**pwr-hh(ii)**pwr
      ro1(i)=ro(i)*difr/(fltd*(a**pwr)*pwr)
      vp1(i)=vp(i)*dif
   10 vs1(i)=vs(i)*dif
c half space scaling
      fact=a/hh(n)
      vp1(n)=vp(n)*fact
      vs1(n)=vs(n)*fact
      ro1(n)=ro(n)*(1.0/fact)**pwr
      z0=0.0
C-----new thicknesses----------------------------
      do 15 i=2,n
      z1=a*alog(a/hh(i))
      h1(i-1)=z1-z0
   15 z0=z1
      h1(n)=0.0
      return
      end
         subroutine init(dx,nlay_deriv,idispr,idispl,kind)
C---------to initialize the program--------------------------
        parameter (nsize=3000,nper=1000)
	common/d/  a(nsize),b(nsize),rho(nsize),d(nsize),qs(nsize)
        common/c/nmax,mmax,kmax,idrop,iedit,ndiv,mode,fact
	 common/o/  c(nper,10),t(nper),ratio(nper,10)
        common/newly/ t1,dt,c1,dc,lstop,iq,istru,cinit
        common/deriv/ nderiv,ndpth,dpth(nsize),dspz(nsize),dsprfi(nsize),
     *  drvz(5,nsize),drvrfi(5,nsize)
        common/log/ KEY_ATTEN,KEY_FLAT,KEY_DERIV,KEY_EIGEN,KEY_EIG_NORM,KEY_EIGEN_DER1,KEY_EIGEN_DER2,KEY_VP,KEY_VS,KEY_TH,KEY_RHO
        common/i_nit/KEY_CINIT
	common/ref/ a_ref(nsize),b_ref(nsize),rho_ref(nsize),d_ref(nsize),qs_ref(nsize),dpth_ref(200)
C-------------------------------------------------------------------------
        logical KEY_DERIV,KEY_RHO,KEY_TH,KEY_VP,KEY_VS,KEY_ATTEN,KEY_FLAT
	logical KEY_EIGEN,KEY_EIG_NORM,KEY_EIGEN_DER1,KEY_EIGEN_DER2,KEY_CINIT
C-------------------------------------------------------------------------
        character*10 symbol,type
        character*1  symb
        character*80 infile,outfile
	character*80 der_phvfile1,der_phvfile2,der_grvfile1,der_grvfile2
        character*80 phvfile,grvfile
        character*80 eigenfileV,eigenfileH
        character*80 attfile
	data ndiv/5/,fact/4.0/,istru/0/,dc/0.01/,iedit/2/,nderiv/1/
	data ndpth/1800/,dpth0/0.0/,ddepth/3.0/,r0/6371.0/,perturb/1./
c       fact - number of wavelenghts below first layer where phase
c       velocity is less than shear velocity that we may consider
c       to be effective halfspace.
C--------------INITIATION------------------------------------------S
        KEY_FLAT=      .FALSE.
        KEY_EIGEN=     .FALSE.
        KEY_EIG_NORM=  .FALSE.
        KEY_EIGEN_DER1=.FALSE.
        KEY_EIGEN_DER2=.FALSE.
	KEY_DERIV=     .FALSE.
        KEY_RHO=       .FALSE.
        KEY_VP=        .FALSE.
        KEY_VS=        .FALSE.
        KEY_TH=        .FALSE.
        KEY_ATTEN=     .FALSE.
	KEY_CINIT=     .FALSE.
        ncount=0
        idispr=0
        idispl=0
c----------------------------------------------------------------------
c       idispl>0 - Love wave calculations 
c       idispr>0 - Rayleigh wave calculations 
c----------------------------------------------------------------------
        print*,'surface wave forward calculations for earthquake seismology problems:'
	print*,'options for flattenning, attenuation, partial derivatives,'
	print*,'eigenfunctions and their derivatives by depth'
C-------interpreting arguments--------------------------------S
        narg=iargc()
	if (narg.lt.8.or.narg.gt.17)then
	print*,'USAGE: SURF_PERT infile  outfile R/L kmin kmax tmin tmax tstep'
	print*,'[-f] [-a] [-s depth_step] [-d par_type nl dx] [-e[1/2/n]] [-c  cinit] [-p perturb]'
	STOP
	endif
	  call GETARG(1,infile)
	  call GETARG(2,outfile)
	  ln=lnblnk(outfile)
        open(1,file=infile,STATUS='OLD')
	  call GETARG(3,symb)
        open(2,file=outfile(1:ln)//'.'//symb)
        if(symb.eq.'L') kind=1
        if(symb.eq.'R') kind=2
        if(symb.eq.'L') idispl=1
        if(symb.eq.'R') idispr=1
	if(symb.ne.'R'.and.symb.ne.'L') STOP' WRONG WAVE TYPE'
	  call GETARG(4,symbol)
	read(symbol,*) k_min
	  call GETARG(5,symbol)
	read(symbol,*) k_max
	  call GETARG(6,symbol)
	read(symbol,*) t1    
	  call GETARG(7,symbol)
	read(symbol,*) tmax
	  call GETARG(8,symbol)
	read(symbol,*) dt
	if(narg.gt.8)  then
        PRint*,'dt=',dt
	                do i=9,narg
	 call GETARG(i,symbol)
	 if (symbol(1:2).eq.'-a')KEY_ATTEN=.true.
	 if (symbol(1:2).eq.'-f')KEY_FLAT=.true.
	 if (symbol(1:2).eq.'-c')then
	      KEY_CINIT=.true.
	      init_c=i+1
				 endif
	 if (symbol(1:2).eq.'-s')then
	      call GETARG(i+1,symbol)
         read(symbol,*)ddepth
				 endif
	 if (symbol(1:2).eq.'-d') then
		KEY_DERIV=.true.
                i_d=i
			         end if  
	 if (symbol(1:2).eq.'-p') then
	  call GETARG(i+1,symbol)
	read(symbol,*) perturb
			         end if  
	 if (symbol(1:2).eq.'-e')then
	        KEY_EIGEN=.true.
	    le=lnblnk(symbol)
	    if(le.eq.2) go to 1111
	    if(symbol(3:3).eq.'1'.or.symbol(3:3).eq.'2') KEY_EIGEN_DER1=.true.
	    if(symbol(3:3).eq.'2') KEY_EIGEN_DER2=.true.
	    if(symbol(3:3).eq.'n') KEY_EIG_NORM=.true.
1111                           endif        
	                      enddo
                  endif
C-------interpreting arguments--------------------------------E
C--------DEFINING and OPENING of OUTPUT FILES------------------------S
C--------for velocities---------------------------------S
           grvfile(1:ln+6)=outfile(1:ln)//'.'//symb//'.grv'
           phvfile(1:ln+6)=outfile(1:ln)//'.'//symb//'.phv'
	   open(21,file=phvfile(1:ln+6))
	   open(22,file=grvfile(1:ln+6))
C--------for velocities---------------------------------E
C--------for initial phase velocity---------------------S
         if(KEY_CINIT) then
	 call GETARG(init_c,symbol)
	 read(symbol,*) cinit
	 print*,KEY_CINIT,'starting phase velocity=',cinit
		       endif
C--------for initial phase velocity---------------------E
C--------for partial derivatives found by numerical technique-S
            if(KEY_DERIV) then
	    call GETARG(i_d+1,type)
	    lt=lnblnk(type)
	    if(type(1:lt).eq.'vs'.or.type(1:lt).eq.'VS')KEY_VS=.true.
	    if(type(1:lt).eq.'vp'.or.type(1:lt).eq.'VP')KEY_VP=.true.
	    if(type(1:lt).eq.'rho'.or.type(1:lt).eq.'RHO')KEY_RHO=.true.
	    if(type(1:lt).eq.'th'.or.type(1:lt).eq.'TH')KEY_TH=.true.
	    call GETARG(i_d+3,symbol)
	    read(symbol,*) dx     
	    call GETARG(i_d+2,symbol)
	    ls=lnblnk(symbol)
	  read(symbol,*) nlay_deriv
	  lqq=ln+lt+ls+12 
	  der_phvfile1(1:lqq)=outfile(1:ln)//'_'//type(1:lt)//'_'//symbol(1:ls)//'.'//symb//'.phv_1st'
	  der_phvfile2(1:lqq)=outfile(1:ln)//'_'//type(1:lt)//'_'//symbol(1:ls)//'.'//symb//'.phv_2nd'
	  der_grvfile1(1:lqq)=outfile(1:ln)//'_'//type(1:lt)//'_'//symbol(1:ls)//'.'//symb//'.grv_1st'
	  der_grvfile2(1:lqq)=outfile(1:ln)//'_'//type(1:lt)//'_'//symbol(1:ls)//'.'//symb//'.grv_2nd'
        open(23,file=der_phvfile1(1:lqq))
        open(24,file=der_phvfile2(1:lqq))
        open(25,file=der_grvfile1(1:lqq))
        open(26,file=der_grvfile2(1:lqq))
                 	endif
C---------to make depths------------------------------S
           dpth(1)=dpth0
	   dpth_ref(1)=dpth(1)
	   do i=2,ndpth
	   dpth(i)=dpth(i-1)+ddepth
	   dpth_ref(i)=dpth(i)
	   enddo
	   if(KEY_FLAT) then
	   do i=1,ndpth
	   dpth_ref(i)=dpth(i)
	   dpth(i)=R0*alog(R0/(R0-dpth(i)))
	   enddo
			    endif
C---------to read depths------------------------------E
C--------for partial derivatives found by numerical technique--E
C--------for eigenfunctions------------------------------------S
	 if(KEY_EIGEN) then
	 eigenfileH=outfile(1:ln)//'.'//symb//'_HOR.eig'
	 lns=lnblnk(eigenfileH)
	 open(27,file=eigenfileH(1:lns))
	 if(symb.eq.'R') then
	 eigenfileV=outfile(1:ln)//'.'//symb//'_VER.eig'
	 lns=lnblnk(eigenfileV)
	 open(28,file=eigenfileV(1:lns))
		      endif
		      endif
C--------for eigenfunctions------------------------------------E
C--------for attenuation--------------------------------------S
         if(KEY_ATTEN) then
	 attfile=outfile(1:ln)//'.'//symb//'.att'
	 lns=lnblnk(attfile)
	 open(29,file=attfile(1:lns))
		       endif
C--------for attenuation---------------------------------------E
C--------DEFINING and OPENING of OUTPUT FILES------------------------E
        if(KEY_EIGEN) iedit=2
	if(KEY_EIGEN_DER2) nderiv=2
C-------reading of the input model into "ref_files"-----S
c----------------------------------------------------------------------
c       d - thickness of layer in kilometers,
c       a - compressional wave velocity in km/sec,
c       b - transvers wave velocity in km/sec,
c       rho - density in gm/sm**3.
c       qs - Q factor for S-wave
c----------------------------------------------------------------------
	do i=1,nsize
        if(KEY_ATTEN) read(1,*,end=99)d_ref(i),a_ref(i),b_ref(i),rho_ref(i),qs_ref(i)
        if(.NOT.KEY_ATTEN) read(1,*,end=99)d_ref(i),a_ref(i),b_ref(i),rho_ref(i)
	enddo
99      mmax=i-1
        if (mmax.lt.2) STOP' LESS THAN 2  LAYERS'
	d(mmax)=0.0
	if(KEY_ATTEN) then
	do i=1,mmax
	if(qs_ref(i).eq.0.0) qs_ref(i)=10000.
	qs_ref(i)=1./qs_ref(i)
	enddo
	             endif
C-------reading of the input model into "ref_files"-----E
        mode=k_max-k_min+1
c-------to define an array of periods--------------------------------S
        kmax=int((tmax-t1)/dt)+1
        PRint*,'kmax=',kmax
	if(kmax.gt.nper) then
	PRINT*,'Too many periods=',kmax,'>',nper
	kmax=nper
		         endif
        PRINT *, 'NLAYERS=', mmax,' NUMB. OF PERIODS=',kmax
        t(1)=t1
                                do i=2,kmax
 	t(i)=t(i-1)+dt
				enddo
CNEW-----------------------------------------------S   
              do i=1,kmax
        t(i)=t(i)*perturb 
              enddo        
CNEW-----------------------------------------------E   
c-------to define an array of periods--------------------------------E
        PRINT*,'    YOUR CHOICE:'
        PRINT*,'DDEPTH=',ddepth,' perturb=',perturb
	IF(symb.EQ.'R')PRINT*,'WAVE TYPE = RAYLEIGH   '
	IF(symb.EQ.'L')PRINT*,'WAVE TYPE = LOVE       '
	IF(KEY_FLAT)PRINT*,'FLATTENNING          '
	IF(KEY_ATTEN)PRINT*,'ANELASTICITY         '
	IF(KEY_EIGEN)PRINT*,'EIGEN FUNCTIONS      '
	IF(KEY_DERIV) then
	PRINT*,'PARTIAL DERIVATIVES for the ',nlay_deriv,'th LAYER'
	IF(KEY_VP)PRINT*,'by VP with perturbation=',dx,'km/s'
	IF(KEY_VS)PRINT*,'by VS with perturbation=',dx,'km/s'
	IF(KEY_RHO)PRINT*,'by RHO with perturbation=',dx,'g/cm**3'
	IF(KEY_TH)PRINT*,'by THICKNESS with perturbation=',dx,'km'
		     endif
        return
	end
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
        subroutine NEVILL (t,c1,c2,del1,del2,ifunc,cc)
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
        parameter (nsize=3000)
        common/d/   a(nsize),b(nsize),rho(nsize),d(nsize),qs(nsize)
        common/c/nmax,mmax,kmax,idrop,iedit,ndiv,mode,fact
        common/newly/t1,dt,cc1,dc,lstop,iqq,istru
        dimension x(20),y(20)
        data lim_cycle/50/
c--------------------------------------------------------------------
        accur1=0.1e-5
        accur2=0.1e-7
        ic=0
        c3=(c1+c2)/2.
        del3=dltar(c3,t,ifunc)
        nev=1
1310    ic=ic+1
             if(ic.lt.lim_cycle)           goto 777
C            if(ic.lt.50)           goto 777
c------------------------------------------------------
c         TOO  MANY  CYCLES        ????? STOP ??????
c------------------------------------------------------
c                                                       STOP
        lstop=lstop+10
	PRINT*,'LSTOP=', lstop,ic
                   goto 2000
c------------------------------------------------------
c
777               continue
             if(c1-c3) 1320,1320,1330
1320         if(c2-c3) 1344,1344,1000
1330         if(c2-c3) 1000,1344,1344
1000    s13=del1-del3
        s32=del3-del2
             if(sign(1.,del3)*sign(1.,del1)) 1441,1441,1443
1441    c2=c3
        del2=del3
                  goto 1444
1443    c1=c3
        del1=del3
1444              continue
             if(abs(c1-c2)-accur1) 20,20,22
22                continue
             if(sign(1.,s13).ne.sign(1.,s32)) nev=0
        ss1=abs(del1)
        s1=0.1*ss1
        ss2=abs(del2)
        s2=0.1*ss2
             if(s1.gt.ss2.or.s2.gt.ss1)           goto 1344
             if(nev.eq.0)           goto 1344
             if(nev.eq.2)           goto 1350
        x(1)=c1
        y(1)=del1
        x(2)=c2
        y(2)=del2
        m=1
                  goto 1355
1344    c3=(c1+c2)/2.
        del3=dltar(c3,t,ifunc)
        nev=1
        m=1
                  goto 1310
1350    x(m+1)=c3
        y(m+1)=del3
1355              do 1360 kk=1,m
        j=m-kk+1
             if(abs(y(m+1)-y(j)).le.accur2)          goto 1344
        x(j)=(-y(j)*x(j+1)+y(m+1)*x(j))/(y(m+1)-y(j))
1360              continue
21                continue
        c3=x(1)
        del3=dltar(c3,t,ifunc)
        nev=2
        m=m+1
             if(m.gt.10) m=10
                  goto 1310
20                continue
        cc=c3
2000              continue
                                                        RETURN
                                                        END
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
        function DLTAR (cc,tt,kk)
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
         parameter (nsize=3000,nper=1000)
        common/d/  a(nsize),b(nsize),rho(nsize),d(nsize),qs(nsize)
        common/o/  c(nper,10),t(nper),ratio(nper,10)
        common/c/nmax,mmax,kmax,idrop,iedit,ndiv,mode,fact
c-----------------------------------------------------------------------
             if(idrop) 899,899,905
899               continue
        dmax=fact*cc*tt
        mmax=nmax
        sum=0
                  do 900 ii=1,nmax
             if(cc-b(ii)) 901,900,900
901     sum=sum+d(ii)
             if(sum-dmax) 900,900,902
902     mmax=ii
             goto 904
900               continue
904     idrop=1
             if(mmax.lt.2) mmax=2
905               continue
             goto (1,2,3,4,5),kk
c---------------------------------------------
c       LOVE  WAVE  PERIOD  equation
c------------------
1       dltar=dltar1(cc,tt,1)
                                                        RETURN
c---------------------------------------------
c       RAYLEIGH  WAVE  PERIOD  equation
c------------------
2       dltar=dltar4(cc,tt,1)
        dlt_old=dltar
						        RETURN	
c---------------------------------------------
c       RAYLEIGH  WAVE  ellipticity
c------------------
3       dltar=dltar4(cc,tt,2)
                                                        RETURN
c---------------------------------------------
c       RAYLEIGH  WAVE  amplitude  response  component
c------------------
4       dltar=dltar4(cc,tt,3)
                                                        RETURN
c---------------------------------------------
c       LOVE  WAVE  amplitude  response  component
c------------------
5       dltar=dltar1(cc,tt,2)
                                                        RETURN
                                                        END
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
        function DLTAR1 (c,t,mup)
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
c-----------------------------------------------------------------------
c       Haskell-Thompson Love wave formulation from halfspace to surface
c-----------------------------------------------------------------------
         parameter (nsize=3000)
        common/d/  a(nsize),b(nsize),rho(nsize),d(nsize),qs(nsize)
        common/c/ nmax,mmax,kmax,idrop,iedit,ndiv,mode,fact
        wvno=6.2831853/(c*t)
        covb=c/b(mmax)
        h=rho(mmax)*b(mmax)*b(mmax)
        rb=sqrt(abs(covb**2-1.))
        ut=1.
        tt=h*rb
        mmm1=mmax-1
                                do 1340 k=1,mmm1
        m=mmax-k
                        if(b(m).eq.0.0)                 go to 1340
        covb=c/b(m)
        rb=sqrt(abs(covb**2-1.))
        h=rho(m)*b(m)*b(m)
7001    q=-wvno*d(m)*rb
                        if(rb.lt.0.1e-20)               go to 1221
                        if(c-b(m)) 1209,1221,1231
1231    sinq=sin(q)
        y=sinq/rb
        z=rb*sinq
        cosq=cos(q)
                                                        go to 1242
1221    y=-wvno*d(m)
        z=0
        cosq=1
                                                        go to 1242
1209    exqp=exp(q)
        exqm=1./exqp
        y=(exqp-exqm)/(2.*rb)
        z=-rb*rb*y
        cosq=(exqp+exqm)/2.
1242    eut=cosq*ut-y*tt/h
        ett=h*z*ut+cosq*tt
        ut=eut
        tt=ett
1340                                    continue
                                                        go to (1,2),mup
1       dltar1=-ett
                                                                RETURN
2       dltar1=ut
                                                                RETURN
                                                                END
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
        function DLTAR4 (c,t,mup)
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
         parameter (nsize=3000)
        common/d/  p(nsize),s(nsize),rho(nsize),d(nsize),qs(nsize)
        common/c/nmax,mmax,kmax,idrop,iedit,ndiv,mode,fact
c-------------------------------------------------------------------
        accur=1.e-8
        accurs=1.e-8
99      wvno=6.28318531/(c*t)
        csq=c*c
        jump=1
             if(mup.gt.1) jump=2
1000    b1=0.0
        b2=0.0
        b3=0.0
        b4=0.0
        b5=0.0
             if(jump-2) 1,2,3
1       b1=1.0
             goto 4
2       b2=1.0
             goto 4
3       b3=1.0
             goto 4
4                 continue
                  do 50 m=1,mmax
100     arga=1.-csq/p(m)**2
        ra=sqrt(abs(arga))
             if(arga.gt.0.) ra=-ra
             if(abs(s(m)).gt.accurs)      goto 101
c---------------------------------------------
c       LIQUID  SURFACE  LAYER
c-------------------------------
        pm=wvno*ra*d(m)
             if(mup.gt.1)      goto 50
        rhoc=rho(m)*csq
             if(abs(ra).lt.accur)      goto 312
             if(ra) 313,312,314
312     sinpr=wvno*d(m)
        rsinp=0.0
        cosp=1.0
             goto 315
313     sinpr=(exp(pm)-exp(-pm))/(2.*ra)
        rsinp=-ra*ra*sinpr
        cosp=0.5*(exp(pm)+exp(-pm))
             goto 315
314     sinpr=sin(pm)/ra
        rsinp=ra*sin(pm)
        cosp=cos(pm)
315               continue
        a11=cosp
        a21=rhoc*sinpr
        a31=0.0
        a41=0.0
        a51=0.0
        a12=0.0
        a22=0.0
        a32=0.0
        a42=0.0
        a13=0.0
        a23=0.0
        a33=0.0
        a14=0.0
        a24=0.0
        a15=0.0
             goto 1001
c-------------------------------------------------
101     argb=1.-csq/s(m)**2
        rb=sqrt(abs(argb))
             if(argb.gt.0.) rb=-rb
        g=2.*s(m)**2/csq
        g1=g-1.
             if(mmax-m) 40,52,40
40      rhoc=rho(m)*csq
        pm=wvno*ra*d(m)
        qm=wvno*rb*d(m)
             if(ra) 213,212,214
212     rsinp=0.0
        sinpr=wvno*d(m)
        cosp=1.0
             goto 215
213     rsinp=-ra*0.5*(exp(pm)-exp(-pm))
        sinpr=-rsinp/(ra**2)
        cosp=0.5*(exp(pm)+exp(-pm))
             goto 215
214     rsinp=ra*sin(pm)
        sinpr=rsinp/(ra**2)
        cosp=cos(pm)
215                 continue
             if(abs(rb).lt.accur)      goto 218
             if(rb.gt.0.)      goto 217
        rsinq=-rb*0.5*(exp(qm)-exp(-qm))
        sinqr=-rsinq/(rb**2)
        cosq=0.5*(exp(qm)+exp(-qm))
             goto 219
217     rsinq=rb*sin(qm)
        sinqr=rsinq/(rb**2)
        cosq=cos(qm)
             goto 219
218     rsinq=0.0
        sinqr=wvno*d(m)
        cosq=1.0
             goto 219
219     rr=rsinp*rsinq
        ss=sinpr*sinqr
        cc=cosp*cosq
        rs1=rsinp*cosq
        rs2=sinqr*cosp
        rs3=sinpr*cosq
        rs4=rsinq*cosp
        gm=2.*g-1
        gs=g*g
        g1s=g1*g1
        ccm=1.-cc
        gg1=g*g1
        rhocs=rhoc*rhoc
        suu=gs*rr+g1s*ss
        a11=2.*gs-gm
        a11=a11*cc-suu-2.*gg1
        a12=-(rs1+rs2)/rhoc
        a13=gm*ccm+g1*ss+g*rr
        a13=-2.*a13/rhoc
        a14=(rs3+rs4)/rhoc
        a15=2.*ccm+rr+ss
        a15=a15/rhocs
        a21=rhoc*(g1s*rs3+gs*rs4)
        a22=cc
        a23=2.*(g*rs4+g1*rs3)
        a24=sinpr*rsinq
        a31=rhoc*(gg1*gm*ccm+g1s*g1*ss+gs*g*rr)
        a32=g1*rs2+g*rs1
        a33=1.+2.*(2*gg1*ccm+suu)
        a41=-rhoc*(g1s*rs2+gs*rs1)
        a42=rsinp*sinqr
        a51=rhocs*(2.*gs*g1s*ccm+gs*gs*rr+g1s*g1s*ss)
1001              continue
c--------------------------------------------------
c         MATRIX  MULTIPLICATION
c               effect  of  immaginary  elements  included
c---------------------------
        bb1=a11*b1+a12*b2+a13*b3+a14*b4+a15*b5
        bb2=a21*b1+a22*b2+a23*b3+a24*b4-a14*b5
        bb3=a31*b1+a32*b2+a33*b3-0.5*a23*b4+0.5*a13*b5
        bb4=a41*b1+a42*b2-2.*a32*b3+a22*b4-a12*b5
        bb5=a51*b1-a41*b2+2.*a31*b3-a21*b4+a11*b5
        b1=bb1
        b2=bb2
        b3=bb3
        b4=bb4
        b5=bb5
50                continue
c--------------------------------------------------
c         the  FOLLOWING  EXPRESSION  is  VALID   rb=0
c------------------------------
52                continue
        pp=p(m)
        sss=s(m)**2
        ppp=pp**2
        rhp=rho(m)*pp
        gra=g*ra
        g1s=g1*g1
        rba=rb-1./ra
        a11=-2.*rb*sss/ppp+csq*g1s/ppp/gra
        a12=rhp*pp
        a13=-rb/a12+g1/a12/gra
        a14=rb/a12/gra
        a15=rba/rhp/rhp/csq/g
        a12=-1./g/a12
        bb1=a11*b1+a12*b2+2.*a13*b3+a14*b4+a15*b5
             goto (501,502,503),mup
501     dltar4=-bb1
                                                        RETURN
502          if(jump.eq.2) r12=bb1
        jump=jump+1
             if(jump.eq.3)      goto 1000
        dltar4=0.5*bb1/r12
                                                        RETURN
503     dltar4=abs(bb1)
             if(abs(s(1)).gt.accurs)                    RETURN
        ra=c/p(1)
        rad=wvno*d(1)*sqrt(abs(ra*ra-1.))
        dltar4=abs(bb1*cos(rad))
                                                        RETURN
                                                        END
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
        subroutine LEIGEN(num,ixa,jxa)
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
c----------------------------------------------------------------------
c       num = 0 - the first call, num > 0 - following calls;
c       ixa - index of period; jxa - index of mode;
c       jmax < 0 or =0 - end of program, jmax > 0 - number of layers;
c       d - layer thickness; a - Pvel; b - Svel; rho - density;
c       depth - depth of the midpoint of layer; xmu,xlamb - Lame const;
c       t - period; c - phase vel; cvar, ugr - phase and group vel 
c       obtained from integrals; wvnum - wave number.
c----------------------------------------------------------------------
         parameter (nsize=3000,nper=1000)
        common/d/  a(nsize),b(nsize),rho(nsize),d(nsize),qs(nsize)
        common/o/ cc(nper,10),tt(nper),rat(nper,10)
        common/c/ nmax,jmax,kmax,idrop,iedit,ndiv,mode,any
        common/rar/ dept1(nsize),amp,ampuz,stresz,stress,mmax
        common/rar1/ dcda,dcdb,dcdr,dwx,g,g2
        common/rco/ c,cvar,ugr,wvno,ratio,ale
        common/rco1/ sumi0,sumi1,sumi2,sumi3,flagr
        common/deriv/ nderiv,ndpth,dpth(nsize),dspz(nsize),dsp(nsize),
     *  drvz(5,nsize),drv(5,nsize)
	common/log/ KEY_ATTEN,KEY_FLAT,KEY_DERIV
        real*8 dcda(nsize),dcdb(nsize),dcdr(nsize),dwx(nsize)
        dimension dmm(5),smm(5),amp(nsize),stress(nsize)
        dimension depth(nsize)
C       dimension xmu(nsize),xlamb(nsize),g(nsize)
        dimension xmu(nsize),g(nsize)
        dimension pr(5*nsize)
        dimension ampuz(nsize),stresz(nsize),g2(nsize)
	data const_lim/1.E+10/,const_lim1/1.E+5/
        xxmin=1.0e-20
c
c
                        if(num.gt.0.and..not.KEY_ATTEN)     go to 503
        mmax=jmax
        nmax=mmax
                        if(mmax) 777,777,778
778                                     continue
        base=0.0
        mm1=mmax-1
        ivre=(nsize-1)/mm1
                        if(ndiv.gt.ivre) ndiv=ivre
        div=float(ndiv)
                        if(ndiv.le.1)                   go to 20003
        jj=1
                        if(b(1).le.0.1e-10) jj=2
                                do 10001 j=jj,mm1
        ldiv=(j-jj)*ndiv
                                do 10001 i=1,ndiv
        pr(ldiv+i)=d(j)/div
        pr(ldiv+i+nsize)=b(j)
        pr(ldiv+i+2*nsize)=rho(j)
	if(KEY_ATTEN)pr(ldiv+i+3*nsize)=qs(j)
10001                                   continue
        mmax=(mm1-jj+1)*ndiv+jj
        d(mmax)=0.
        a(mmax)=a(nmax)
        b(mmax)=b(nmax)
        rho(mmax)=rho(nmax)
	if(KEY_ATTEN)qs(mmax)=qs(nmax)
        nmax=mmax
        mm1=mmax-1
                                do 10002 j=jj,mm1
        d(j)=pr(j-jj+1)
        b(j)=pr(j-jj+nsize+1)
        rho(j)=pr(j-jj+2*nsize+1)
        if(KEY_ATTEN)qs(j)=pr(j-jj+3*nsize+1)
10002                                   continue
20003                                   continue
c
c
c
c
c
                                do 22 i=1,mmax
        base=base+d(i)
        depth(i)=base-d(i)*0.5
        xmu(i)=rho(i)*b(i)*b(i)
                        if(i-mmax)22,24,24
22                                      continue
24      depth(mmax)=base-d(mmax)
c
c
503                                     continue
                                do 500 j=1,nsize
        amp(j)=0.0
        stress(j)=0.0
        dcdr(j)=0.0
        dcdb(j)=0.0
500     g(j)=0.0
        t=tt(ixa)
        c=cc(ixa,jxa)
499                                     continue
        mmax=nmax
c-----------------------------------------------------------------------
c       Layer dropping procedure; any is the same as fact.
c-----------------------------------------------------------------------
                        if(any.le.0.) any=7.0
        dmax=any*c*t
899     sum=0.
                                do 900 ii=1,mmax
        max=ii
                        if(c-b(ii)) 901,900,900
901     sum=sum+d(ii)
                        if(ii.eq.mmax)                  go to 902
                        if(sum.le.dmax)                 go to 900
                        if(b(ii+1)-b(ii)) 902,900,90009
900                                     continue
90009   max=max+1
902     mmax=max
        wvno=6.2831853/(c*t)
        wvnum=wvno
        wvnosq=wvno*wvno
        omegsq=(6.2831853/t)**2
C-------the halfspace is handled-------------------------
        ut0=1.
7777    ut=ut0
        covb=c/b(mmax)
        h=rho(mmax)*b(mmax)*b(mmax)
        rb=wvno*sqrt(abs(covb**2-1.))
        tq=-h*rb*ut0
        amp(mmax)=ut
        stress(mmax)=tq
        g(mmax)=tq/xmu(mmax)
                        if(rb) 1001,1000,1001
1000    dm=1.0e25
        sm=0.
                                                        go to 1002
1001    dm=0.5/rb
        sm=0.5*rb
1002                                    continue
        dldr=omegsq*dm
        dldm=-(wvnosq*dm+sm)
        dcdb(mmax)=2.*rho(mmax)*b(mmax)*c*dldm/wvno
        dcdr(mmax)=(c/wvno)*(dldr+b(mmax)*b(mmax)*dldm)
        mmm1=mmax-1
        sumi0=rho(mmax)*dm
        sumi1=h*dm
        sumi2=h*sm
C---------------loop for knots-----------S
                                do 1340 k=1,mmm1
 	IF(abs(ut).gt.const_lim) THEN
         ut0=ut0/const_lim1
 	goto 7777 
 				  ENDIF
        m=mmax-k
                        if(b(m).eq.0.0)                 go to 1340
        covb=c/b(m)
        rb=wvno*sqrt(abs(covb**2-1.))
        h=rho(m)*b(m)*b(m)
        dz=d(m)/4.
        dmm(1)=ut*ut
        smm(1)=(tq/h)**2
                                do 1339 kk=2,5
        xkk=kk-1
        q=rb*dz*xkk
                        if(c-b(m)) 1207,1221,1231
1231    sinq=sin(q)
        y=sinq/rb
        z=-rb*sinq
        cosq=cos(q)
                                                        go to 1242
1221    y=dz*xkk
        z=0.0
        cosq=1.0
                                                        go to 1242
1207    exqp=exp(q)
        exqm=1./exqp
        y=(exqp-exqm)/(2.*rb)
        z=rb*rb*y
        cosq=(exqp+exqm)/2.
1242    eut=cosq*ut-y*tq/h
        ett=-h*z*ut+cosq*tq
        dmm(kk)=eut*eut
        smm(kk)=(ett*ett)/(h*h)
                        if(kk-3) 1339,1301,1339
1301    amp(m)=eut
        stress(m)=ett
        g(m)=ett/xmu(m)
1339                                    continue
        ut=eut
        tq=ett
        dm=(dz/22.5)*(7.*(dmm(1)+dmm(5))+32.*(dmm(2)+dmm(4))+12.*dmm(3))
        sm=(dz/22.5)*(7.*(smm(1)+smm(5))+32.*(smm(2)+smm(4))+12.*smm(3))
        dldm=-(wvnosq*dm+sm)
        dldr=omegsq*dm
        dcdb(m)=2.*rho(m)*b(m)*c*dldm/wvno
        dcdr(m)=(c/wvno)*(dldr+b(m)*b(m)*dldm)
        sumi0=sumi0+rho(m)*dm
        sumi1=sumi1+h*dm
        sumi2=sumi2+h*sm
1340                                    continue
C---------------loop for knots-----------E
        amp(nsize)=1.0
        stress(nsize)=tq/ut
        dcdb(nsize)=0.0
        dcdr(nsize)=0.0
        depth(nsize)=0.0
                        if(b(1).eq.0.0) depth(nsize)=d(1)
        dldk=-2.*wvno*sumi1
                                do 1500 l=1,mmax
                        if(b(l).eq.0.0)                 go to 1500
        amp(l)=amp(l)/ut
        stress(l)=stress(l)/ut
        dcdb(l)=dcdb(l)/dldk
        dcdr(l)=dcdr(l)/dldk
        g(l)=stress(l)/xmu(l)
c--------------------------------------------------------
c       Exclusion of low amplitudes from final output
c--------------------------------------------------------
                        if(b(l)-b(mmax)) 1506,1507,1507
1507                                    continue
                        if(abs(amp(l))-xxmin) 1501,1502,1502
1501    amp(l)=0.0
        stress(l)=0.0
        dcdb(l)=0.0
        dcdr(l)=0.0
        g(l)=0.0
                                                        go to 1500
1502                                    continue
1506                                    continue
1500                                    continue
        sumi0=sumi0/(ut*ut)
        sumi1=sumi1/(ut*ut)
        sumi2=sumi2/(ut*ut)
        wvar=(omegsq*sumi0-sumi2)/sumi1
        cvar=sqrt(omegsq/wvar)
        wvar=sqrt(wvar)
        ugr=sumi1/(c*sumi0)
        flagr=omegsq*sumi0-wvnosq*sumi1-sumi2
        ale=1./(2.*c*ugr*sumi0)/sqrt(6.28318)*1.e-15
                        if(b(1).le.0.0)                 go to 5556
                                do 5555 l=1,mmax
        i=mmax-l+1
        j=i+1
        dept1(j)=depth(i)
        amp(j)=amp(i)
        stress(j)=stress(i)
        dcdb(j)=dcdb(i)
        dcdr(j)=dcdr(i)
5555    g(j)=g(i)
        mmax=mmax+1
                                                        go to 501
5556                            do 5557 l=1,mmax
5557    dept1(l)=depth(l)
501     dept1(1)=0.
                        if(b(1).le.0.0) dept1(1)=d(1)
        amp(1)=1.
        stress(1)=0.
        dcdb(1)=0.
        dcdr(1)=0.
        g(1)=0.
c----------------------------------------------------------------------
c       Calculation of eigenfunction and its derivatives
c----------------------------------------------------------------------
                        if(iedit.ne.2)                  go to 800
                                do 799 i=1,5
                                do 799 j=1,nsize
        dsp(j)=0.0
        drv(i,j)=0.0
799                                     continue
                                do 801 l=1,ndpth
                        if(dpth(l).lt.dept1(1))         go to 801
c----------------------------------------------------------------------
c       If dpth(l) is above the free surface, the output will be zero
c----------------------------------------------------------------------
                                do 802 ii=2,mmax
        rab1=dept1(ii)-dpth(l)
                        if(abs(rab1).lt.1.0e-5) rab1=0.
                        if(rab1.le.0.)                  go to 802
        i=ii-1
        j=ii
                        if(b(1).le.0.0) i=ii
        rab2=d(i)/2.
        rab1=rab1-rab2
                        if(abs(rab1).lt.1.0e-5) rab1=0.
                        if(rab1.le.0.)                  go to 803
        j=j-1
        i=i-1
                                                        go to 803
802                                     continue
        j=mmax
        i=mmax-1
                        if(b(1).le.0.0) i=j
803     dz=dpth(l)-dept1(j)
        ampl=amp(j)
        stres=stress(j)
        covb=c/b(i)
        rb=wvno*sqrt(abs(covb**2-1.))
                        if(c-b(i)) 810,811,812
810     nderv=nderiv+1
                                do 804 ii=1,nderv
        k=ii-1
        rab1=rb**k
        q=rb*dz
        expq=exp(q)
        uttq=(ampl+stres/(xmu(i)*rb))/ampl
        rab2=rab1*expq*uttq*ampl/2.
        uttq=abs(uttq)
                        if(uttq.lt.1.0e-5) rab2=0.
        rab3=(-1)**k*rab1*(ampl/2.-stres/(2.*xmu(i)*rb))/expq
                        if(k.ne.0)                      go to 805
        dsp(l)=rab2+rab3
                                                        go to 804
805     drv(k,l)=rab2+rab3
804                                     continue
                                                        go to 801
811     dsp(l)=ampl+stres*dz/xmu(i)
                        if(nderiv.lt.1)                 go to 801
        drv(1,l)=stres/xmu(i)
                                                        go to 801
812     nderv=nderiv+1
                                do 806 ii=1,nderv
        k=ii-1
        rab1=rb**k
        q=rb*dz
        q=q+k*3.1415926/2.
        cosq=cos(q)
        sinq=sin(q)
        rab2=rab1*ampl*cosq
        rab3=stres*rab1*sinq/(xmu(i)*rb)
                        if(k.ne.0)                      go to 807
        dsp(l)=rab2+rab3
                                                        go to 806
807     drv(k,l)=rab2+rab3
806                                     continue
801                                     continue
800                                                             RETURN
777                                     continue
                                                                STOP
                                                                END
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
        subroutine REIGEN (num,ixa,jxa)
czzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
         parameter (nsize=3000,nper=1000)
        double precision yy1,yy2,yy3,yy4,yz1,yz2,yz3,yz4,aur1,auz1,
     1  atz1,atr1,aur2,auz2,atz2,atr2,aa,bb,daur1,dauz1,datz1,datr1,
     2  daur2,dauz2,datz2,datr2,eaur1,eauz1,eatz1,eatr1,eaur2,eauz2,
     3  eatz2,eatr2,saur1,sauz1,satz1,satr1,saur2,sauz2,satz2,satr2,
     4  xnorm,dmmr,dmmz,smmz,smmr,drsz,dzsr,dldr,dldm,dldl,dldk,
     5  dcdr(nsize),dcda(nsize),dcdb(nsize),dwx(nsize)
        complex cra,cr1,cr2,cr3
        common/d/   a(nsize),b(nsize),rho(nsize),d(nsize),qs(nsize)
        common/o/   cc(nper,10),tt(nper),rat(nper,10)
        common/c/   nmax,jmax,kmax,idrop,iedit,ndiv,mode,fact 
        common/newly/ te1,dete,ce1,dece,lstop,iqq,istru
        common/rar/ dept1(nsize),ampur,ampuz,stresz,stresr,mmax
        common/rar1/ dcda,dcdb,dcdr,dwx,g1,g2
        common/rco/ c,cvar,ugr,wvno,ratio,are
        common/rco1/ sumi0,sumi1,sumi2,sumi3,flagr
        common/deriv/ nderiv,ndpth,dpth(nsize),dspz(nsize),dspr(nsize),
     *  drvz(5,nsize),drvr(5,nsize)
	common/log/ KEY_ATTEN,KEY_FLAT,KEY_DERIV
        dimension   stresr(nsize),stresz(nsize),
     *  wwt(4),wt(4)
        dimension yy1(nsize,5),yy2(nsize,5),yy3(nsize,5),yy4(nsize,5)
        dimension yz1(nsize,5),yz2(nsize,5),yz3(nsize,5),yz4(nsize,5)
        dimension dmrsmz(5),dmzsmr(5),depth(nsize),ampur(nsize),ampuz(nsize)
        dimension dmr(5),dmz(5),smr(5),smz(5),g1(nsize),g2(nsize)
        dimension pr(5*nsize),xlamb(nsize),xmu(nsize)
c----------------------------------------------------------------------
                  do 99999 i=1,nsize
                  do 99999 j=1,5
        yy1(i,j)=1.
        yy2(i,j)=1.
        yy3(i,j)=1.
        yy4(i,j)=1.
        yz1(i,j)=2.
        yz2(i,j)=1.
        yz3(i,j)=1.
        yz4(i,j)=1.
99999             continue
        xxmin=1.0e-15
             if(num.gt.0.and..not.KEY_ATTEN)      goto 503
        mmax=jmax
        nmax=mmax
        wwt(1)=0.
        wwt(2)=0.5
        wwt(3)=0.5
        wwt(4)=1.0
        wt(1)=1./6.
        wt(2)=1./3.
        wt(3)=1./3.
        wt(4)=1./6.
             if(fact.le.0) fact=4.
             if(num.eq.-1)      goto 88888
             if(mmax) 777,777,778
c
  778             continue
             if(istru.lt.1)     goto 20005
        call strut
             goto 20004
20005             continue
        base=0.0
        mm1=mmax-1
        ivre=(nsize-1)/mm1
             if(ndiv.gt.ivre) ndiv=ivre
        div=float(ndiv)
        jj=1
             if(ndiv.le.1)      goto 20003
             if(b(1).le.0.1e-10) jj=2
                  do 10001 j=jj,mm1
        ldiv=(j-jj)*ndiv
                  do 10001 i=1,ndiv
        pr(ldiv+i)=d(j)/div
        pr(ldiv+nsize+i)=a(j)
        pr(ldiv+2*nsize+i)=b(j)
        pr(ldiv+3*nsize+i)=rho(j)
	if(KEY_ATTEN) pr(ldiv+4*nsize+i)=qs(j)
10001             continue
        mmax=(mm1-jj+1)*ndiv+jj
        d(mmax)=0.
        a(mmax)=a(nmax)
        b(mmax)=b(nmax)
        rho(mmax)=rho(nmax)
	if(KEY_ATTEN)qs(mmax)=qs(nmax)
        nmax=mmax
        mm1=mmax-1
                  do 10002 j=jj,mm1
        d(j)=pr(j-jj+1)
        a(j)=pr(j-jj+nsize+1)
        b(j)=pr(j-jj+2*nsize+1)
        rho(j)=pr(j-jj+3*nsize+1)
	if(KEY_ATTEN) qs(j)=pr(j-jj+4*nsize+1)
10002             continue
20003             continue
c
c
20004             continue
        mmax=nmax
88888   base=0.0
                  do 22 i=1,mmax
        base=base+d(i)
        depth(i)=base-d(i)*0.5
        xmu(i)=rho(i)*b(i)*b(i)
        xlamb(i)=rho(i)*(a(i)*a(i)-2.*b(i)*b(i))
             if(i-mmax) 22,24,24
22                continue
24      depth(mmax)=base-d(mmax)
c
c
503               continue
                  do 500 j=1,nmax
        dcda(j)=0.0
        dcdb(j)=0.0
        dcdr(j)=0.0
        ampur(j)=0.0
        ampuz(j)=0.0
        stresr(j)=0.0
        stresz(j)=0.0
        g1(j)=0.0
        g2(j)=0.0
500               continue
        t=tt(ixa)
        c=cc(ixa,jxa)
        ratio=rat(ixa,jxa)
        mmax=nmax
        dmax=fact*t*c
899     sum=0.0
                  do 900 ii=1,mmax
        max=ii
             if(c-b(ii)) 901,900,900
901     sum=sum+d(ii)
             if(ii.eq.mmax)      goto 902
             if(sum.le.dmax)      goto 900
             if(a(ii+1)-a(ii)) 902,903,90009
903          if(b(ii+1)-b(ii)) 902,900,90009
900               continue
90009   max=max+1
902     mmax=max
        sumi0=0.0
        sumi1=0.0
        sumi2=0.0
        sumi3=0.0
        wvno=6.2831853072/(c*t)
        wvnosq=wvno*wvno
        wvnum=wvno
        omega=6.2831853072/t
        omegsq=omega*omega
             if(b(1).gt.0.0)      goto 4000
        ra=c/a(1)
        cr1=ra*ra-1.
        cra=wvno*csqrt(cr1)
                        if(cabs(cra).le.1.0e-35)        go to 4001
                                                        go to 4002
4001    sumi0=rho(1)*d(1)
        sumi1=0.0
        sumi2=0.0
        sumi3=0.0
        tzz=0.0
             goto 4000
4002    cr1=csin(2.*cra*d(1))
        cr2=4.*cra
        cr3=cr1/cr2
        sin2ra=real(cr3)
        cr1=ccos(cra*d(1))
        cosra=real(cr1)
        cos2rm=1./(cosra*cosra)
        fac1=(0.5*d(1)+sin2ra)*cos2rm
        fac3=wvno*(0.5*d(1)-sin2ra)*cos2rm
        cr1=cra*cra
        rab1=real(cr1)
        fac2=wvno*fac3/(rab1)
        fac4=rab1*fac3/wvno
        sumi0=rho(1)*(fac1+fac2)
        sumi1=xlamb(1)*fac2
        sumi2=xlamb(1)*fac3
        sumi3=xlamb(1)*fac4
        cr1=csin(cra*d(1))
        cr2=cr1/cra
        rab1=real(cr2)
        tzz=-rho(1)*omegsq*rab1/cosra
4000              continue
        cova=c/a(mmax)
        covb=c/b(mmax)
        gam=2./covb**2
        gamm1=gam-1.
        ra=wvno*sqrt(abs(cova**2-1.))
        rb=wvno*sqrt(abs(covb**2-1.))
        det=wvnosq-ra*rb
        h=rho(mmax)*omegsq
        brkt=-gamm1*wvno+gam*ra*rb/wvno
        iter=0
        aur1=1.d+00
        auz1=0.d+00
        atz1=-h*brkt/det
        atr1=-h*ra/det
        mmm1=mmax-1
                  do 1346 mm=1,mmm1
        m=mmax-mm
             if(b(m).le.0.0)      goto 1346
        xdiv=1.
        ddz=-d(m)/(4.*xdiv)
        a12=1./(xlamb(m)+2.*xmu(m))
        a13=wvno*xlamb(m)*a12
        a21=-omegsq*rho(m)
        a24=wvno
        a31=-wvno
        a34=1./xmu(m)
        a42=-a13
        a43=a21+4.*wvnosq*xmu(m)*(xlamb(m)+xmu(m))*a12
        yy3(m,5)=aur1
        yy1(m,5)=auz1
        yy2(m,5)=atz1
        yy4(m,5)=atr1
                  do 1338 kk=2,5
        k=6-kk
        eaur1=aur1
        eauz1=auz1
        eatz1=atz1
        eatr1=atr1
        daur1=0.d00
        dauz1=0.d00
        datz1=0.d00
        datr1=0.d00
                  do 1401 ll=1,4
        saur1=aur1+wwt(ll)*ddz*daur1
        sauz1=auz1+wwt(ll)*ddz*dauz1
        satz1=atz1+wwt(ll)*ddz*datz1
        satr1=atr1+wwt(ll)*ddz*datr1
        daur1=a31*sauz1+a34*satr1
        dauz1=a12*satz1+a13*saur1
        datz1=a21*sauz1+a24*satr1
        datr1=a42*satz1+a43*saur1
        eaur1=eaur1+wt(ll)*ddz*daur1
        eauz1=eauz1+wt(ll)*ddz*dauz1
        eatz1=eatz1+wt(ll)*ddz*datz1
        eatr1=eatr1+wt(ll)*ddz*datr1
1401              continue
        aur1=eaur1
        auz1=eauz1
        atz1=eatz1
        atr1=eatr1
1400              continue
        yy1(m,k)=auz1
        yy2(m,k)=atz1
        yy3(m,k)=aur1
        yy4(m,k)=atr1
1338              continue
1346              continue
             if(b(1).gt.0.0)      goto 1347
        yy1(1,1)=yy1(2,1)
        yy2(1,1)=yy2(2,1)
        yy3(1,1)=yy3(2,1)
        yy4(1,1)=yy4(2,1)
1347              continue
4003    aur2=0.d00
        auz2=1.d00
        atz2=-h*rb/det
        atr2=-h*brkt/det
             if(iter.eq.0)      goto 4004
        aur1=1.d 00
        auz1=0.d 00
        atz1=-h*brkt/det
        atr1=-h*ra/det
        aur2=aur2+xnorm*aur1
        auz2=auz2+xnorm*auz1
        atz2=atz2+xnorm*atz1
        atr2=atr2+xnorm*atr1
4004              do 2346 mm=1,mmm1
        m=mmax-mm
             if(b(m).le.0.0)      goto 2346
        ddz=-d(m)/(4.*xdiv)
        a12=1./(xlamb(m)+2.*xmu(m))
        a13=wvno*xlamb(m)*a12
        a21=-omegsq*rho(m)
        a24=wvno
        a31=-wvno
        a34=1./xmu(m)
        a42=-a13
        a43=a21+4.*wvnosq*xmu(m)*(xlamb(m)+xmu(m))*a12
        yz3(m,5)=aur2
        yz1(m,5)=auz2
        yz2(m,5)=atz2
        yz4(m,5)=atr2
                do 2338 kk=2,5
        k=6-kk
        eaur2=aur2
        eauz2=auz2
        eatz2=atz2
        eatr2=atr2
        daur2=0.0
        dauz2=0.0
        datz2=0.0
        datr2=0.0
                  do 2401 ll=1,4
        saur2=aur2+wwt(ll)*ddz*daur2
        sauz2=auz2+wwt(ll)*ddz*dauz2
        satz2=atz2+wwt(ll)*ddz*datz2
        satr2=atr2+wwt(ll)*ddz*datr2
        daur2=a31*sauz2+a34*satr2
        dauz2=a12*satz2+a13*saur2
        datz2=a21*sauz2+a24*satr2
        datr2=a42*satz2+a43*saur2
        eaur2=eaur2+wt(ll)*ddz*daur2
        eauz2=eauz2+wt(ll)*ddz*dauz2
        eatz2=eatz2+wt(ll)*ddz*datz2
        eatr2=eatr2+wt(ll)*ddz*datr2
2401              continue
        aur2=eaur2
        auz2=eauz2
        atz2=eatz2
        atr2=eatr2
2400              continue
        yz1(m,k)=auz2
        yz2(m,k)=atz2
        yz3(m,k)=aur2
        yz4(m,k)=atr2
2338              continue
2346              continue
             if(b(1).gt.0.0)      goto 2347
        yz1(1,1)=yz1(2,1)
        yz2(1,1)=yz2(2,1)
        yz3(1,1)=yz3(2,1)
        yz4(1,1)=yz4(2,1)
2347              continue
        aa=yz3(1,1)-ratio*yz1(1,1)
        bb=ratio*yy1(1,1)-yy3(1,1)
             if(dabs(bb).lt.1.d-10) bb=dsign(1.d-10,bb)
        xnorm=aa/bb
        bb=xnorm*yy1(1,1)+yz1(1,1)
             if(dabs(bb).lt.1.d-10) bb=dsign(1.d-10,bb)
        ampur(nsize)=(xnorm*yy3(1,1)+yz3(1,1))/bb
        ampuz(nsize)=(xnorm*yy1(1,1)+yz1(1,1))/bb
        stresz(nsize)=(xnorm*yy2(1,1)+yz2(1,1))/bb
        stresr(nsize)=(xnorm*yy4(1,1)+yz4(1,1))/bb
        iter=iter+1
             if(iter.gt.1)      goto 2011
        xtest=abs(ampur(nsize)/ratio-1.)
             if(xtest.ge.0.00001)      goto 4003
2011    dcdb(nsize-1)=iter
        dcda(nsize)=0.0
        dcdb(nsize)=0.0
        dcdr(nsize)=0.0
        ampur(nsize-1)=yy3(1,1)
        ampur(nsize-2)=yz3(1,1)
        ampuz(nsize-1)=yy1(1,1)
        ampuz(nsize-2)=yz1(1,1)
        stresz(nsize-1)=yy2(1,1)
        stresz(nsize-2)=yz2(1,1)
        stresr(nsize-1)=yy4(1,1)
        stresr(nsize-2)=yz4(1,1)
        dcdb(nsize-2)=bb
        dcda(nsize-2)=0.0
        dcda(nsize-1)=0.0
        dcdr(nsize-2)=0.0
        dcdr(nsize-1)=0.0
                  do 7000 m=1,mmax
             if(b(m).le.0.0)      goto 7000
             if(m-mmax) 7001,77777,77777
7001    dz=d(m)/4.
                  do 1339 kk=1,5
        aur=(xnorm*yy3(m,kk)+yz3(m,kk))/bb
        auz=(xnorm*yy1(m,kk)+yz1(m,kk))/bb
        atz=(xnorm*yy2(m,kk)+yz2(m,kk))/bb
        atr=(xnorm*yy4(m,kk)+yz4(m,kk))/bb
        durdz=atr/xmu(m)-wvno*auz
        duzdz=(atz+wvno*xlamb(m)*aur)/(xlamb(m)+2.*xmu(m))
        dmr(kk)=aur*aur
        dmz(kk)=auz*auz
        smr(kk)=durdz*durdz
        smz(kk)=duzdz*duzdz
        dmrsmz(kk)=aur*duzdz
        dmzsmr(kk)=auz*durdz
             if(kk-3) 1339,1301,1339
1301    ampur(m)=aur
        ampuz(m)=auz
        stresz(m)=atz
        stresr(m)=atr
        g1(m)=(atz+aur*xlamb(m)*wvno)/(xlamb(m)+2.*xmu(m))
        g2(m)=atr/xmu(m)
1339              continue
        dmmr=(dz/22.5)*(7.*(dmr(1)+dmr(5))+32.*(dmr(2)+dmr(4))+12.
     *  *dmr(3))
        dmmz=(dz/22.5)*(7.*(dmz(1)+dmz(5))+32.*(dmz(2)+dmz(4))+12.
     *  *dmz(3))
        smmz=(dz/22.5)*(7.*(smz(1)+smz(5))+32.*(smz(2)+smz(4))+12.
     *  *smz(3))
        smmr=(dz/22.5)*(7.*(smr(1)+smr(5))+32.*(smr(2)+smr(4))+12.
     *  *smr(3))
        drsz=(dz/22.5)*(7.*(dmrsmz(1)+dmrsmz(5))+32.*(dmrsmz(2)
     *  +dmrsmz(4))+12.*dmrsmz(3))
        dzsr=(dz/22.5)*(7.*(dmzsmr(1)+dmzsmr(5))+32.*(dmzsmr(2)
     *  +dmzsmr(4))+12.*dmzsmr(3))
        sumi0=sumi0+rho(m)*(dmmr+dmmz)
        sumi1=(xlamb(m)+2.*xmu(m))*dmmr+xmu(m)*dmmz+sumi1
        sumi2=xmu(m)*dzsr-xlamb(m)*drsz+sumi2
        sumi3=(xlamb(m)+2.*xmu(m))*smmz+xmu(m)*smmr+sumi3
        dldl=-wvnosq*dmmr+2.*wvno*drsz-smmz
        dldm=-wvnosq*(2.*dmmr+dmmz)-2.*wvno*dzsr-(2.*smmz+smmr)
        dldr=omegsq*(dmmr+dmmz)
        dcdb(m)=2.*rho(m)*b(m)*c*(dldm-2.*dldl)/wvno
        dcda(m)=2.*rho(m)*a(m)*c*dldl/wvno
        dcdr(m)=(c/wvno)*(dldr+xlamb(m)*dldl/rho(m)+xmu(m)*dldm/rho(m))
             if(abs(auz)+abs(aur)-xxmin) 7002,7002,7000
7000              continue
77777             continue
             if((b(1).gt.0.1e-10).or.m.ne.2)      goto 7002
        aur=ratio
        auz=1.
        atr=0.
        atz=tzz
7002    ampur(m)=aur
        ampuz(m)=auz
        stresr(m)=atr
        stresz(m)=atz
        g1(m)=(atz+aur*xlamb(m)*wvno)/(xlamb(m)+2.*xmu(m))
        g2(m)=atr/xmu(m)
        ap=-rho(m)*(wvno*aur+rb*auz)/det
        bp=-rho(m)*(-ra*aur/wvno-auz)/det
        a1=-wvno*ap/rho(m)
        a2=-wvno*rb*bp/rho(m)
        a3=ra*ap/rho(m)
        a4=wvnosq*bp/rho(m)
             if(rb) 7005,7006,7005
7005    dmmr=a1*a1/(2.*ra)+2.*a1*a2/(ra+rb)+a2*a2/(2.*rb)
        dmmz=a3*a3/(2.*ra)+2.*a3*a4/(ra+rb)+a4*a4/(2.*rb)
        smmz=ra*a3*a3/2.+2.*ra*rb*a3*a4/(ra+rb)+rb*a4*a4/2.
        smmr=ra*a1*a1/2.+2.*ra*rb*a1*a2/(ra+rb)+rb*a2*a2/2.
        drsz=-a1*a3/2.-(a1*a4*rb+a2*a3*ra)/(ra+rb)-a2*a4/2.
        dzsr=-a1*a3/2.-(a1*a4*ra+a2*a3*rb)/(ra+rb)-a2*a4/2.
             goto 7010
7006    ugr=b(m)
        flagr=0.0
        sumi0=rho(m)*10.**(25)
        sumi1=xmu(m)*10.**(25)
        sumi2=0.0
        sumi3=0.0
        are=0.0
        dcdb(m)=-2.*wvno*10.**(25)
             goto 531
7010              continue
        sumi0=sumi0+rho(m)*(dmmr+dmmz)
        sumi1=(xlamb(m)+2.*xmu(m))*dmmr+xmu(m)*dmmz+sumi1
        sumi2=xmu(m)*dzsr-xlamb(m)*drsz+sumi2
        sumi3=(xlamb(m)+2.*xmu(m))*smmz+xmu(m)*smmr+sumi3
        dldr=omegsq*(dmmr+dmmz)
        dldm=-wvnosq*(2.*dmmr+dmmz)-2.*wvno*dzsr-(2.*smmz+smmr)
        dldl=-wvnosq*dmmr+2.*wvno*drsz-smmz
        dcda(m)=2.*rho(m)*a(m)*c*dldl/wvno
        dcdb(m)=2.*rho(m)*b(m)*c*(dldm-2.*dldl)/wvno
        dcdr(m)=(c/wvno)*(dldr+xlamb(m)*dldl/rho(m)+xmu(m)*dldm/rho(m))
7011              continue
        ugr=(wvno*sumi1+sumi2)/(omega*sumi0)
        flagr=omegsq*sumi0-wvnosq*sumi1-2.*wvno*sumi2-sumi3
        wvar=(-sumi2+sqrt(abs(sumi2**2-sumi1*(sumi3-
     *  omegsq*sumi0))))/sumi1
        cvar=omega/wvar
        are=1./(2.*c*ugr*sumi0)/sqrt(6.28318)*1.e-15
531               continue
501               continue
c
c
c
c
c
c
        n=1
             if(b(1).le.0.0) n=2
                  do 5010 m=n,mmax
        dldk=-2.*(wvnum*sumi1+sumi2)
        dcdr(m)=dcdr(m)/dldk
        dcda(m)=dcda(m)/dldk
        dcdb(m)=dcdb(m)/dldk
        dwx(m)=(dcda(m)*4./3.*b(m)/a(m)+dcdb(m))*b(m)
5010                continue
                        if(b(1).le.0.0)         go to 5556
        do 5555 m=1,mmax
        i=mmax-m+1
        j=i+1
        dept1(j)=depth(i)
        ampur(j)=ampur(i)
        ampuz(j)=ampuz(i)
        stresz(j)=stresz(i)
        stresr(j)=stresr(i)
        dcda(j)=dcda(i)
        dcdb(j)=dcdb(i)
        dcdr(j)=dcdr(i)
        dwx(j)=dwx(i)
        g1(j)=g1(i)
        g2(j)=g2(i)
5555                continue
        mmax=mmax+1
                                                go to 5558
5556                            do 5557 m=1,mmax
5557    dept1(m)=depth(m)
5558    dept1(1)=0.
        if(b(1).le.0.0) dept1(1)=d(1)
        ampur(1)=ratio
        ampuz(1)=1.
        stresz(1)=0.
        if(b(1).le.0.0) stresz(1)=tzz
        stresr(1)=0.
        dcda(1)=0.
        dcdb(1)=0.
        dcdr(1)=0.
        dwx(1)=0.
        g1(1)=0.
        g2(1)=0.
c----------------------------------------------------------------------
c       Calculation of eigen function and its derivatives
c----------------------------------------------------------------------
                        if(iedit.ne.2)                  go to 800
                                do 799 i=1,5
                                do 799 j=1,nsize
        dspz(j)=0.0
        dspr(j)=0.0
        drvz(i,j)=0.0
        drvr(i,j)=0.0
799                                     continue
                                do 801 l=1,ndpth
                        if(dpth(l).lt.dept1(1))         go to 801
c----------------------------------------------------------------------
c       If dpth(l) is above the free surface, the output will be zero
c----------------------------------------------------------------------
                                do 802 ii=2,mmax
        rab1=dept1(ii)-dpth(l)
                        if(abs(rab1).lt.1.0e-5) rab1=0.
                        if(rab1.le.0.)                  go to 802
        i=ii-1
        j=ii
                        if(b(1).le.0.0) i=ii
        rab2=d(i)/2.
        rab1=rab1-rab2
                        if(abs(rab1).lt.1.0e-5) rab1=0.
                        if(rab1.le.0.)                  go to 803
        j=j-1
        i=i-1
                                                        go to 803
802                                     continue
        j=mmax
        i=mmax-1
                        if(b(1).le.0.0) i=j
803     dz=dpth(l)-dept1(j)
        amplz=ampuz(j)
        amplr=ampur(j)
        strsz=stresz(j)
        strsr=stresr(j)
        cova=c/a(i)
        covb=c/b(i)
        rb=wvno*sqrt(abs(covb**2-1.))
        ra=wvno*sqrt(abs(cova**2-1.))
        del=2.*b(i)*b(i)-c*c
c----------------------------------------------------------------
c       Calculation of terms, connected with a-velocity
c----------------------------------------------------------------
        rab1=(c-a(i))/c
                        if(abs(rab1).lt.1.0e-5) rab1=0.
                        if(rab1) 810,811,812
810     nderv=nderiv+1
        q=ra*dz
        expq=exp(q)
        rab2=2.*xmu(i)*wvno*ra*amplr
        rab3=del*rho(i)*wvno*wvno*amplz
        rab4=2.*rho(i)*c*c*ra*wvno
        c1=(rab2-rab3+wvno*strsr-ra*strsz)/rab4
        uttq=abs(c1/amplr)
                        if(uttq.lt.1.0e-5) c1=0.
        c2=(rab2+rab3-wvno*strsr-ra*strsz)/rab4
                                do 804 ii=1,nderv
        k=ii-1
        rab1=ra**k
        rab2=rab1*c1*expq
        rab3=(-1)**k*rab1*c2/expq
                        if(k.ne.0)                      go to 805
        dspr(l)=rab2+rab3
        dspz(l)=(ra*rab2-ra*rab3)/wvno
                                                        go to 804
805     drvr(k,l)=rab2+rab3
        drvz(k,l)=(ra*rab2-ra*rab3)/wvno
804                                     continue
                                                        go to 700
811     rab2=(2.*xmu(i)*wvno*amplr-strsz)/(rho(i)*c*c*wvno)
        rab3=(del*rho(i)*wvno*amplz-strsr)/(rho(i)*c*c)
        dspr(l)=rab2-rab3*dz
        dspz(l)=-rab3/wvno
                        if(nderiv.lt.1)                 go to 700
        drvr(1,l)=-rab3
        drvz(1,l)=0.
                                                        go to 700
812     nderv=nderiv+1
        c1=(2.*xmu(i)*wvno*amplr-strsz)/(rho(i)*c*c*wvno)
        c2=(del*rho(i)*wvno*amplz-strsr)/(rho(i)*c*c*wvno)
                                do 806 ii=1,nderv
        k=ii-1
        rab1=ra**k
        q=ra*dz+k*3.1415926/2.
        cosq=cos(q)
        sinq=sin(q)
                        if(k.ne.0)                      go to 807
        dspr(l)=c1*cosq-wvno*c2*sinq/ra
        dspz(l)=-ra*c1*sinq/wvno-c2*cosq
                                                        go to 806
807     drvr(k,l)=(c1*cosq-wvno*c2*sinq/ra)*rab1
        drvz(k,l)=-(ra*c1*sinq/wvno+c2*cosq)*rab1
806                                     continue
c----------------------------------------------------------------------
c       Calculation of terms, connected with b-velocity
c----------------------------------------------------------------------
700     rab1=(c-b(i))/c
                        if(abs(rab1).lt.1.0e-5) rab1=0.
                        if(rab1) 710,711,712
710     nderv=nderiv+1
        q=rb*dz
        expq=exp(q)
        rab2=2.*xmu(i)*wvno*rb*amplz
        rab3=del*rho(i)*wvno*wvno*amplr
        rab4=2.*rho(i)*c*c*wvno*wvno
        c3=(rab2-rab3+wvno*strsz-rb*strsr)/rab4
        uttq=abs(c3/amplz)
                        if(uttq.lt.1.0e-5) c3=0.
        c4=(-rab2-rab3+wvno*strsz+rb*strsr)/rab4
                                do 704 ii=1,nderv
        k=ii-1
        rab1=rb**k
        rab2=rab1*c3*expq
        rab3=(-1)**k*rab1*c4/expq
                        if(k.ne.0)                      go to 705
        dspr(l)=dspr(l)+rab2+rab3
        dspz(l)=dspz(l)+(rab2-rab3)*wvno/rb
                                                        go to 704
705     drvr(k,l)=drvr(k,l)+rab2+rab3
        drvz(k,l)=drvz(k,l)+(rab2-rab3)*wvno/rb
704                                     continue
                                                        go to 801
711     rab2=(2.*xmu(i)*wvno*amplz-strsr)/(rho(i)*c*c*wvno)
        rab3=(del*rho(i)*wvno*amplr-strsz)/(rho(i)*c*c)
        dspz(l)=dspz(l)+rab2-rab3*dz
        dspr(l)=dspr(l)-rab3/wvno
                        if(nderiv.lt.1)                 go to 801
        drvz(1,l)=drvz(1,l)-rab3
                                                        go to 801
712     nderv=nderiv+1
        c3=(2.*xmu(i)*wvno*amplz-strsr)/(rho(i)*c*c*wvno)
        c4=(del*rho(i)*wvno*amplr-strsz)/(rho(i)*c*c*wvno)
                                do 706 ii=1,nderv
        k=ii-1
        rab1=rb**k
        q=rb*dz+k*3.1415926/2.
        cosq=cos(q)
        sinq=sin(q)
                        if(k.ne.0)                      go to 707
        dspz(l)=dspz(l)+c3*cosq-wvno*c4*sinq/rb
        dspr(l)=dspr(l)-rb*c3*sinq/wvno-c4*cosq
                                                        go to 706
707     drvz(k,l)=drvz(k,l)+(c3*cosq-wvno*c4*sinq/rb)*rab1
        drvr(k,l)=drvr(k,l)-(rb*c3*sinq/wvno+c4*cosq)*rab1
706                                     continue
801                                     continue
800                                                             RETURN
777                    continue
                                                                STOP
                                                                END
c///////////////////////////////////////////////////////////////////
c-------------------------------------------------------------------
        subroutine STRUT
c-------------------------------------------------------------------
        parameter (nsize=3000)
        common/st/ nst,accur,ds
        common/d/ a(nsize),b(nsize),rho(nsize),d(nsize),qs(nsize)
        common/c/ nmax,q1,q2,q3,q4,q5,q6,q7
        dimension ds(nsize),r(nsize),rr(nsize)
c-------------------------------------------------------------------
c
c    ---- NST GT 0 ---- ---- IF B(1)=0   NMAX GT 0 --------
c
c-------------------------------------------------------------------
        d(nmax)=0.
             if(b(1).ge.0.1e-10)      goto 100
                                      do 1  i=1,nst
             if(ds(i).ge.d(1))      goto 100
        ds(i)=d(1)
1                                     continue
100                         continue
        mmax=nmax-1
             if(mmax.eq.0)      goto 101
        r(1)=d(1)
             if(mmax.eq.1)      goto 101
                                      do 2  i=2,mmax
        j=i-1
        r(i)=r(j)+d(i)
2                                     continue
101                         continue
        rr(1)=0.
        l=1
        m=1
        n=1
                                      do 3  i=1,98
             if(n.gt.nst)      goto 103
             if(m.gt.mmax)      goto 102
        h=ds(n)-r(m)
             if(h.ge.accur)      goto 103
102                         continue
        n=n+1
             if(l.gt.1)      goto 104
             if(abs(ds(n-1)).le.accur)      goto 3
             goto 106
104                         continue
        f=ds(n-1)-abs(rr(l))
             if(abs(f).le.accur)      goto 3
106                         continue
             if(m.gt.mmax)      goto 107
             if(abs(h).le.accur)      goto 103
107                         continue
        l=l+1
        rr(l)=ds(n-1)
             goto 3
103                         continue
             if(m.gt.mmax)      goto 105
        l=l+1
        rr(l)=-r(m)
        m=m+1
3                                     continue
105                         continue
        k=nmax
        rr(l+1)=rr(l)
                                      do 4  i=1,l
        j=l-i+1
        a(j)=a(k)
        b(j)=b(k)
        rho(j)=rho(k)
             if(rr(j).ge.0.0)      goto 108
        rr(j)=-rr(j)
        k=k-1
108                         continue
        d(j)=rr(j+1)-rr(j)
        
4                                     continue
        nmax=l
                                                        RETURN
                                                        END



